# TEXTRACT -- Extract TRES image into one or two echelle files
# March 16, 2009
# By Doug Mink, SAO Telescope Data Center

procedure textract (images)

file	images=""	{prompt="Input image(s) to extract"}
file	aproot=""	{prompt="Root filename for aperature reference spectrum"}
file	apdir="tresdata$"	{prompt="Directory for reference spectra"}
bool	recenter=yes	{prompt="Recenter orders (yes or no)"}
bool	redo=no		{prompt="Delete existing output files and re-extract (yes or no)"}
string	orders="1-51"	{prompt="Orders to read"}
string	readnoise="1"	{prompt="Read out noise sigma (photons)"}
string	gain="1"	{prompt="Photon gain (photons/data number)"}
real    npeaks=0.2	{prompt="Select brightest peaks"}
bool	interactive=no	{prompt="Run apall interactively (y or n)"}
bool	review=no	{prompt="Review results of extraction (y or n)"}
bool	deltemp=yes	{prompt="Delete temporary file (y or n)"}
file	spec1=""	{prompt="Output spectrum from fiber 1, if any"}
file	spec2=""	{prompt="Output spectrum from fiber 2, if any"}
file	spec3=""	{prompt="Output spectrum from fiber 3, if any"}
bool	verbose=yes	{prompt="Log progress (y or n)"}
bool	debug=no	{prompt="Print debugging information (y or n)"}

	begin
	bool vb, db
	file imin, imfib1, imfib2, imfib3, imroot, templist, listfile
	file outfib1, outfib2, outfib3, imref, dbref
	int lroot
	string history, ims, tchar, args, fib1, fib2, fib3, fibkey, fibki

	vb = verbose
	db = debug

# If input image is a list prepend @ if not already there
	ims = images
        smatch (ims, ".list")
        if (smatch.found) {
            if (substr (ims,1,1) != '@')
		ims = "@" // images
            }

# Set up extraction parameters
	textract1.aproot = aproot
	textract1.apdir =apdir
	textract1.recenter = recenter
	textract1.orders = orders
	textract1.readnoise = readnois
	textract1.gain = gain
	textract1.redo = redo
	textract1.npeaks = npeaks
	textract1.interactive = interactive
	textract1.review = review
	textract1.verbose = vb
	textract1.debug = db

#  Make temporary list of input files
	tchar = substr (ims,1,1)
	templist = mktemp ("tempext") // ".list"
        if (tchar == '@') {
	    listfile = substr (ims,2,100)
	    copy (listfile, templist)
	    }
	else {
	    files (ims, >templist)
	    }
	list = templist
	while (fscan (list,imin) != EOF) {
	    textract1.spec1 = ""
	    textract1.spec2 = ""
	    textract1.spec3 = ""
	    textract1 (imin)
	    }

	if (deltemp)
	    delete (templist)

end

# Sep 13 2007	New task based on hectospec.hextract
# Sep 18 2007	Allow extraction of any of the three apertures of data
# Sep 21 2007	Use apall instead of apsum and recenter
# Sep 24 2007	Change aperture templates names to root||apsize||apnum
# Sep 25 2007	Add binning parameter
# Sep 25 2007	Change aperture template names to root||(b)||apsize||apnum
# Sep 25 2007	Add recenter parameter so extraction without shift is possible
# Oct  3 2007	Drop fibs, binning, and apsize parameters: read from files
# Oct  3 2007	Set up aperture reference images in tres.tapref
# Oct 11 2007	Do not use file names fro tapref; generate them here
# Dec 19 2007	Default data directory to tresdata$

# Jan 16 2008	Change tresfib to trsfib
# Jan 16 2008	Pass debug and verbose to tapref
# Jan 25 2008	Set other FIBi keywords to OFF in each fiber output spectrum
# Jan 28 2008	Call apref for each fiber
# Jan 30 2008	Change arguments for tapref
# Feb  8 2008	Write output spectrum file names to parameters speci
# Feb 11 2008	Change temporary file from texlist* to tempext*list
# Feb 12 2008	Add specdir="" to trsfib call
# Feb 13 2008	Return without extracting if extracted spectra already exist
# Apr 10 2008	Name single fiber output files _fn rather than _fibn
# May 16 2008	Add input time argument to gettime() 
# Jun  6 2008	Call TRSFIB1 instead of TRSFIB
# Dec 15 2008	Add timediff argument to GETTIME to get current time
# Dec 31 2008	Make @ optional if image is *.list file

# Feb  6 2009	Add overwrite to delete existing output files
# Mar 16 2009	Change overwrite parameter to redo
# Mar 16 2009	Call textract1 to do the extraction
