# TEXTRACT1 -- Extract one TRES image into one or two echelle files
# By Doug Mink, SAO Telescope Data Center
# April 10, 2009

procedure textract1 (image)

	file	image=""	{prompt="Input image to extract"}
	file	aproot=""	{prompt="Root filename for aperature reference spectrum"}
	file	apdir="tresdata$"	{prompt="Directory for reference spectra"}
	bool	recenter=yes	{prompt="Recenter orders (yes or no)"}
	string	orders="1-51"	{prompt="Orders to read"}
	string	readnoise="1"	{prompt="Read out noise sigma (photons)"}
	string	gain="1"	{prompt="Photon gain (photons/data number)"}
	real    npeaks=0.2	{prompt="Select brightest peaks"}
	bool	redo=no		{prompt="Delete existing output files and re-extract (yes or no)"}
	bool	interactive=no	{prompt="Run apall interactively (y or n)"}
	bool	review=no	{prompt="Review results of extraction (y or n)"}
	file	spec1=""	{prompt="Output spectrum from fiber 1, if any"}
	file	spec2=""	{prompt="Output spectrum from fiber 2, if any"}
	file	spec3=""	{prompt="Output spectrum from fiber 3, if any"}
	bool	verbose=yes	{prompt="Log progress (y or n)"}
	bool	debug=no	{prompt="Print debugging information (y or n)"}

	begin
	bool vb, db, fcosmic, icosmic, rd
	file imin, imfib1, imfib2, imfib3, imroot, imref
	file outfib1, outfib2, outfib3
	int lroot, lfk
	string history, ims, fib1, fib2, fib3, fibkey

	imin = image
	vb = verbose
	db = debug
	rd = redo
	spec1 = ""
	spec2 = ""
	spec3 = ""

	if (!access ("database/"))
	    mkdir ("database")

# Make output file name
	smatch (imin, ".ec")
	if (smatch.found) {
	    lroot = smatch.start - 1
	    imroot = substr (imin, 1, lroot)
	    }
	else {
	    smatch (imin, ".fits")
	    if (smatch.found) {
		lroot = smatch.start - 1
		imroot = substr (imin, 1, lroot)
		}
	    else {
		imroot = imin
		}
	    }
	imin = imroot // ".fits"

# Make sure this is not already an extracted image
	imgetp.default = "0"
	imgetp (imin, "TEXTRACT")
	if (imgetp.value != "0") {
	    if (vb) {
		print ("TEXTRACT1: ", imin, " is already an extracted image")
		print ("TEXTRACT = ", imgetp.value)
		}
	    return
	    }

# Set switch if cosmic rays have been removed from the input image
	imgetp.value = ""
	imgetp (imin, "TCOSMIC")
	if (imgetp.value != "")
	    icosmic = yes
	else
	    icosmic = no

# Make output file names for the fibers which are exposed
	tfib1 (imin, specdir="")
	if (tfib1.fib1 != "OFF")
	    outfib1 = imroot // "_f1.ec.fits"
	else
	    outfib1 = ""
	if (tfib1.fib2 != "OFF" && tfib1.fib3 == "OFF")
	    outfib2 = imroot // "_f2.ec.fits"
	else
	    outfib2 = ""
	if (tfib1.fib3 != "OFF")
	    outfib3 = imroot // "_f3.ec.fits"
	else
	    outfib3 = ""
	if (db) {
	    if (outfib1 != "" && outfib2 != "")
		print ("TEXTRACT1: ", imin, " -> ", outfib1," and ", outfib2)
	    else if (outfib1 != "" && outfib3 != "")
		print ("TEXTRACT1: ", imin, " -> ", outfib1," and ", outfib3)
	    else if (outfib1 != "")
		print ("TEXTRACT1: ", imin, " -> ", outfib1)
	    else if (outfib2 != "")
		print ("TEXTRACT1: ", imin, " -> ", outfib2)
	    else if (outfib3 != "")
		print ("TEXTRACT1: ", imin, " -> ", outfib3)
	    }

# Return without extracting if extracted spectra already exist
	imgetp.value = ""
	if (outfib1 != "" && outfib2 != "" && outfib3 != "") {
	    if (access (outfib1) && access (outfib2) && access (outfib3)) {
		imgetp (outfib1, "TCOSMIC")
		if (imgetp.value != "")
		    fcosmic = yes
		else
		    fcosmic = no
		if (rd) {
		    delete (outfib1)
		    delete (outfib2)
		    delete (outfib3)
		    }
		else if ((icosmic && fcosmic) || (!icosmic && !fcosmic)) {
		    spec1 = outfib1
		    spec2 = outfib2
		    spec3 = outfib3
		    if (vb)
			print ("TEXTRACT1: Fibers 1, 2, and 3 have already been extracted")
		    return
		    }
		else {
		    delete (outfib1)
		    delete (outfib2)
		    delete (outfib3)
		    }
		}
	    }
	else if (outfib1 != "" && outfib2 != "") {
	    if (access (outfib1) && access (outfib2)) {
		imgetp (outfib1, "TCOSMIC")
		if (imgetp.value != "")
		    fcosmic = yes
		else
		    fcosmic = no
		if (rd) {
		    delete (outfib1)
		    delete (outfib2)
		    }
		else if ((icosmic && fcosmic) || (!icosmic && !fcosmic)) {
		    spec1 = outfib1
		    spec2 = outfib2
		    if (vb)
			print ("TEXTRACT1: Fibers 1 and 2 have already been extracted")
		    return
		    }
		else {
		    delete (outfib1)
		    delete (outfib2)
		    }
		}
	    }
	else if (outfib3 != "") {
	    if (access (outfib3)) {
		imgetp (outfib3, "TCOSMIC")
		if (imgetp.value != "")
		    fcosmic = yes
		else
		    fcosmic = no
		if (rd) {
		    delete (outfib3)
		    }
		else if ((icosmic && fcosmic) || (!icosmic && !fcosmic)) {
		    spec3 = outfib3
		    if (vb)
			print ("TEXTRACT1: Fiber 3 has already been extracted")
		    return
		    }
		else {
		    delete (outfib3)
		    }
		}
	    }


# Get file names for aperture reference images
	if (aproot == "")
	    aproot = "flat"
	tapref.aproot = aproot
	tapref.apdir = apdir
	tapref.verbose = vb
	tapref.debug = db
	tapref.apsize = tfib1.apsize
	tapref.binning = tfib1.binning

# Set up apall parameters
	apall.apertures = orders
	apall.format = "echelle"
	apall.profiles = ""
	apall.interactive = interactive
	apall.find = no
	apall.recenter = recenter
	apall.resize = no
	apall.edit = no
	apall.trace = no
	apall.fittrace = no
	apall.extract = yes
	apall.extras = no
	apall.review = review
	apall.line = INDEF
	apall.nsum = 10
	apall.lower = -5.
	apall.upper = 5.
	apall.apidtable = ""
	apall.b_function = "legendre"
	apall.b_order = 1
	apall.b_sample = "-10:-6,6:10"
	apall.b_naverage = -3
	apall.b_niterate = 3
	apall.b_low_reject = 3.
	apall.b_high_reject = 3.
	apall.b_grow = 0.
	apall.width = 5.
	apall.radius = 10.
	apall.threshold = 0.
	apall.minsep = 5.
	apall.maxsep = 1000.
	apall.order = "increasing"
	apall.aprecenter = ""
	apall.npeaks = 20
	apall.shift = recenter
	apall.llimit = INDEF
	apall.ulimit = INDEF
	apall.ylevel = 0.1
	apall.peak = yes
	apall.bkg = yes
	apall.r_grow = 0.
	apall.avglimits = no
	apall.t_nsum = 10
	apall.t_step = 10
	apall.t_nlost = 3
	apall.t_function = "legendre"
	apall.t_order = 3
	apall.t_sample = "*"
	apall.t_naverage = 1
	apall.t_niterate = 0
	apall.t_low_reject = 3.
	apall.t_high_reject = 3.
	apall.t_grow = 0.
	apall.background = "none"
	apall.skybox = 1
	apall.weights = "none"
	apall.pfit = "fit1d"
	apall.clean = no
	apall.saturation = INDEF
	apall.readnoise = "0."
	apall.gain = "1."
	apall.lsigma = 4.
	apall.usigma = 4.
	apall.nsubaps = 1

# Set time of operation
	gettime (0.0, timediff=99)
	history = gettime.date // " " // gettime.time // " extracted from " // imin
	hedit.show = vb
	spec1 = ""
	spec2 = ""
	spec3 = ""

# Fiber 1 (usually object)
	imgetp (imin, "FIBKEY")
	fibkey = imgetp.value
	lfk = strlen (fibkey)

# Extract fiber 1 object (usually) spectrum to echelle file
	if (outfib1 != "") {
	    tapref ("", fiber=1)
	    apall.references = tapref.imref
	    print ("TEXTRACT1: Extracting ",outfib1)
	    apall (imin, output=outfib1)
	    hedit (outfib1, "FIBER", "1",  add+, verify-)
	    if (lfk < 3)
		fib1 = substr (fibkey, 1, 1) // "N"
	    else
		fib1 = substr (fibkey, 1, 1) // "NN"
	    hedit (outfib1, "FIBKEY", fib1, add+, verify-)
	    hedit (outfib1, "FIB1", tfib1.fib1, add+, verify-)
	    hedit (outfib1, "FIB2", "OFF", add+, verify-)
	    hedit (outfib1, "FIB3", "OFF", add+, verify-)
	    hedit (outfib1, "BANDID2", "echelle order", add+, verify-)
	    hedit (outfib1, "TEXTRACT", history, add+, verify-)
	    imref = "flat" // tfib1.apsize // tfib1.binning // "1.fits"
	    if (tapref.imref == imref) {
		imgetp.default = imref
		imgetp (imref, "FLATFILE")
		imref = imgetp.value
		}
	    else
		imref = tapref.imref
	    hedit (outfib1, "TEXAPREF", imref, add+, verify-)
	    spec1 = outfib1
	    }

# Extract fiber 2 (usually) background spectrum to echelle file
	if (outfib2 != "") {
	    tapref ("",fiber=2)
	    apall.references = tapref.imref
	    print ("TEXTRACT1: Extracting ",outfib2)
	    apall (imin, output=outfib2)
	    hedit (outfib2, "FIBER", "2",  add+, verify-)
	    if (lfk < 3)
		fib2 = "N" // substr (fibkey, 2, 2)
	    else
		fib2 = "N" // substr (fibkey, 2, 2) // "N"
	    hedit (outfib2, "FIBKEY", fib2, add+, verify-)
	    hedit (outfib2, "FIB1", "OFF", add+, verify-)
	    hedit (outfib2, "FIB2", tfib1.fib2, add+, verify-)
	    hedit (outfib2, "FIB3", "OFF", add+, verify-)
	    hedit (outfib2, "BANDID2", "echelle order", add+, verify-)
	    hedit (outfib2, "TEXTRACT", history, add+, verify-)
	    imref = "flat" // tfib1.apsize // tfib1.binning // "2.fits"
	    if (tapref.imref == imref) {
		imgetp.default = imref
		imgetp (imref, "FLATFILE")
		imref = imgetp.value
		}
	    else
		imref = tapref.imref
	    hedit (outfib2, "TEXAPREF", imref, add+, verify-)
	    spec2 = outfib2
	    }

# Extract fiber 3 calibration spectrum to echelle file
	if (outfib3 != "") {
	    tapref ("", fiber=3)
	    apall.references = tapref.imref
	    print ("TEXTRACT1: Extracting ",outfib3)
	    apall (imin, output=outfib3)
	    hedit (outfib3, "FIBER", "3",  add+, verify-)
	    fib3 = "NN" // substr (fibkey, 3, 3)
	    hedit (outfib3, "FIBKEY", fib3, add+, verify-)
	    hedit (outfib3, "FIB1", "OFF", add+, verify-)
	    hedit (outfib3, "FIB2", "OFF", add+, verify-)
	    hedit (outfib3, "FIB3", tfib1.fib3, add+, verify-)
	    hedit (outfib3, "BANDID2", "echelle order", add+, verify-)
	    hedit (outfib3, "TEXTRACT", history, add+, verify-)
	    imref = "flat" // tfib1.apsize // tfib1.binning // "3.fits"
	    if (tapref.imref == imref) {
		imgetp.default = imref
		imgetp (imref, "FLATFILE")
		imref = imgetp.value
		}
	    else
		imref = tapref.imref
	    hedit (outfib3, "TEXAPREF", imref, add+, verify-)
	    spec3 = outfib3
	    }

end

# Sep 13 2007	New task based on hectospec.hextract
# Sep 18 2007	Allow extraction of any of the three apertures of data
# Sep 21 2007	Use apall instead of apsum and recenter
# Sep 24 2007	Change aperture templates names to root||apsize||apnum
# Sep 25 2007	Add binning parameter
# Sep 25 2007	Change aperture template names to root||(b)||apsize||apnum
# Sep 25 2007	Add recenter parameter so extraction without shift is possible
# Oct  3 2007	Drop fibs, binning, and apsize parameters: read from files
# Oct  3 2007	Set up aperture reference images in tres.tapref
# Oct 11 2007	Do not use file names fro tapref; generate them here
# Dec 19 2007	Default data directory to tresdata$

# Jan 16 2008	Change tresfib to trsfib
# Jan 16 2008	Pass debug and verbose to tapref
# Jan 25 2008	Set other FIBi keywords to OFF in each fiber output spectrum
# Jan 28 2008	Call apref for each fiber
# Jan 30 2008	Change arguments for tapref
# Feb  8 2008	Write output spectrum file names to parameters speci
# Feb 11 2008	Change temporary file from texlist* to tempext*list
# Feb 12 2008	Add specdir="" to trsfib call
# Feb 13 2008	Return without extracting if extracted spectra already exist
# Mar 20 2008	Write FIBi keyword for current fiber in single fiber files
# Apr 10 2008	Drop unused parameter deltemp
# May  1 2008	Add FIBER keyword for single fiber output files
# May  2 2008	Extract if existing fiber spectra still have cosmic rays
# May 20 2008	Fix bug which caused false FIBER header keyword values
# May 23 2008	Save extracted spectrum names as parameters if they already exist
# Jun  6 2008	Call TRSFIB1 instead of TRSFIB
# Aug 13 2008	Name files _f1 _f2 _f3 instead of _fib1 _fib2 _fib3
# Aug 13 2008	Use TFIB1 instead of TRSFIB1
# Dec 15 2008	Add timediff argument to GETTIME to get current time
# Dec 18 2008	Resolve actual aperture reference file if flat<apfib>.fits

# Mar 16 2008	Add redo parameter
# Mar 17 2009	Make sure that all HEDIT parameters are set
# Apr 10 2009	Fix FIBKEY reassignments for 2-fiber system
