# File tres/Scripts/ttres.cl
# April 10, 2009
# By Doug Mink, SAO Telescope Data Center

# TTRES - Process multiple similar TRES ThAr images from raw data files to spectra

procedure ttres (images)

string	images=""	{prompt="Images to process"}
string	datedir=""	{prompt="Date for images"}
string	instdir=""	{prompt="Directory containing date directory"}
bool	fixpreamp=no	{prompt="Run tpreamp to fix time constant (yes or no)"}
bool	fixpix=yes	{prompt="Fix bad pixels (yes or no)"}
bool	fixbias=no	{prompt="Fix bias section (yes or no)"}
real	gaincorr=1.0	{prompt="Gain correction for amp2 to amp1"}
bool	flatten=yes	{prompt="Run TFLAT to Flatten image (yes or no)"}
string	normfile=""	{prompt="Pixel to pixel tmakeflat normalization file"}
string	normdir=""	{prompt="Directory for normalization file"}
real	threshold=10.0	{prompt="Cosmic ray threshold in sigma"}
char	sumspec="no"	{prompt="Process summed 2-D file after tcosmic yes no only"}
string	aproot="flat"	{prompt="Root file name for aperture reference file"}
file	apdir="tresdata$" {prompt="Directory for aperture reference file"}
bool	recenter=no	{prompt="Recenter orders during extraction (yes or no)"}
bool	extract=yes	{prompt="Extract orders to echelle file (yes or no)"}
bool	disperse=yes	{prompt="Dispersion correct extracted spectra (yes or no)"}
bool	medshift=no	{prompt="yes: median shift no: mean shift over orders"}
file	compstd="first"	{prompt="Name of file in list to use as shift standard"}
bool	compid=yes	{prompt="Interactvely ecidentify standard if yes; else run ecreid"}
string	comproot="comp"	{prompt="Root file name for calibration lamp file"}
file	compdir="tresdata$"	{prompt="Directory for calibration lamp file"}
bool	deltemp=yes	{prompt="Delete intermediate image files (y or n)"}
bool	saveraw=yes	{prompt="Save unprocessed image files (y or n)"}
bool	saveold=no	{prompt="Save intermediate image files (y or n)"}
bool	verbose=yes	{prompt="Print process progress (y or n)"}

begin

file	imroot, imin, impath
file	imout, ims, cdir, tempin, tempout
string	ddir, idir, croot, cstd, cnum
int	pl, n
bool	vb, dt, sr, sv, disp, rc, sum, sumonly, cid 
real	ct

	vb = verbose
	dt = deltemp
	sv = saveold
	sr = saveraw
	rc = recenter
	ddir = datedir
	idir = instdir
	ims = images
	cdir = compdir
	croot = comproot
	disp = disperse
	ct = threshold
	cid = compid

	if (ims == "") {
	    print ("TTRES: Reduce multiple similar TRES ThAr spectra, removing cosmic rays")
	    print ("Usage: ttres images with both amps to be corrected")
	    return
	    }

# If input image is a list prepend @ if not already there
        smatch (ims, ".list")
        if (smatch.found) {
            if (substr (ims,1,1) != '@')
		ims = "@" // ims
            }

	tcheck
	if (tcheck.rawdir)
	    trsdate (tcheck.datedir)

#  Make list file from input file parameter
	trsfiles.instdir = idir
	trsfiles.datedir = ddir
	trsfiles.copyfile = yes
	trsfiles.overwrite = no
	trsfiles (ims)
	if (trsfiles.nfiles == 0) {
	    delete (trsfiles.tempin)
	    if (vb)
		print ("TTRES: No files found: ", ims)
	    return
	    }
	tempin = trsfiles.tempin
	tempout = trsfiles.tempout
	ims = "@" // tempout

# Convert reference spectrum from sequence number to filename if necessary
	cstd = compstd
	if (cstd != "") {
	    isnum (cstd)
	    if (isnum.fpcode > 0) {
		n = int (cstd)
		if (n < 10)
		    cnum = "000" // n
		else if (n < 100)
		    cnum = "00" // n
		else if (n < 1000)
		    cnum = "0" // n
		else
		    cnum = "0" // n
		grep (cnum, tempout) | scan (cstd)
		filename (cstd)
		cstd = filename.filename
		}
	    }

# Remove overscan, trim, and merge amplifiers of image
# Then remove pixel to pixel variation from image (including IR fringing)
# Then extract spectra from image and add dispersion function
	tproc.gaincorr = gaincorr
	tproc.fixbias = fixbias
	tproc.fixpix = fixpix
	tproc.fixpreamp = fixpreamp
	tproc.flatten = flatten
	tproc.cosmic = yes
	tproc.sumspec = sumspec
	tproc.threshold = ct
	tproc.extract = extract
	tproc.disperse = disp
	tproc.normfile = normfile
	tproc.normdir = normdir
	tproc.aproot = aproot
	tproc.apdir = apdir
	tproc.medshift = medshift
	tproc.refroot = croot
	tproc.refdir = cdir
	tproc.recenter = rc
	tproc.deltemp = dt
	tproc.saveraw = sr
	tproc.saveold = sv
	tproc.verbose = vb
	tcosmic.savebp = sv
	tproc (ims, compstd=cstd, compid=cid)

#  Delete temporary list files
	if (dt) {
	    delete (tempin)
	    delete (tempout)
	    }

	if (tcheck.rawdir)
	    tdata (datedir=tcheck.datedir)

end

# Jul 17 2008	New task for processing ThAr spectra based on CTRES
# Sep 19 2008	Add medshift parameter which gets passed through TPROC to TCAL1
# Oct 14 2008	Do not run if in raw data directory
# Oct 14 2008	Make default to save raw files
# Oct 15 2008	Add threshold parameter to set cosmic ray threshold
# Oct 16 2008	Add compstd parameter for file name to become comp*ec.fits
# Oct 16 2008	From raw directory, switch to quick-look directory and back
# Dec  5 2008	Make @ optional if image is *.list file
# Dec 18 2008	Accept ThAr reference spectrum as sequence number or file name
# Mar  6 2009	Add compid to allow interacive line ID
# Mar  9 2009	Return if no files found from list
# Apr 10 2009	Make default normdir = "" (working directory)
