# File tres/Src/t_trscode.x
# By Doug Mink
# May 20, 2008

# TRSCODE -- Turn apertures, binning, and aperture size into a single string

include <error.h>
include <imhdr.h>
include <imio.h>
include <ctype.h>

procedure t_trscode ()

char spectrum[SZ_FNAME]	# File to process
char specdir[SZ_PATHNAME] # Input directory for data files
char indir[SZ_PATHNAME]	# Input directory for data files

char apfib[8]		# Returned fiber size (s m l) binning (b) ap (1 2 3)
char fibcode[8]		# TRES aperture size (s=1.8, m=2.3, l=2.9)
char fibsize[8]		# TRES aperture size (1.8, 2.3, 2.9)
int binning		# Binning (1 or 2)
char apertures[8]	# TRES apertures(s) (1 2 3 12 123)
bool vb			# Log progress (yes or no)
bool db			# Print diagnostic information (yes or no)
int ldir, mode
pointer im, im1
char imf[SZ_PATHNAME], imf0[SZ_PATHNAME], imf1[SZ_PATHNAME]
char ccdsum[8]
char fibkey[8]
char fiber[8]

pointer immap()
bool clgetb()
int stridx()
int strlen()
int strcmp()

begin

	vb = clgetb ("verbose")
	db = clgetb ("debug")
	apertures[1] = EOS

# Set up input file path if file name is not null string
        call clgstr ("spectrum", spectrum, SZ_FNAME)
	if (strlen (spectrum) > 0) {
	    call clgstr ("specdir", specdir, SZ_PATHNAME)
	    call strcpy (specdir, indir, SZ_PATHNAME)
	    ldir = strlen (indir)
	    if (ldir < 1)
		call strcpy (spectrum, imf, SZ_PATHNAME)
	    else {
		if (indir[ldir] != '/') {
		    indir[ldir+1] = '/'
		    indir[ldir+2] = EOS
		    }
		call strcpy (indir, imf, SZ_PATHNAME)
		call strcat (spectrum, imf, SZ_PATHNAME)
		}
	    call strcpy (imf, imf0, SZ_PATHNAME)
	    call strcat ("[0]", imf0, SZ_PATHNAME)
	    if (db) {
		call printf ("TRESFIB: reading %s\n")
		    call pargstr (imf0)
		call flush (STDOUT)
		}

# Open image file to read the header
	    mode = READ_ONLY
	    iferr (im = immap (imf0, mode, 0)) {
		if (im != NULL)
		    call imunmap (im)
		if (db) {
		    call printf ("TRESFIB: image file %s read error\n")
			call pargstr (imf0)
		    call flush (STDOUT)
		    }
		return
		}
	    else if (db) {
		call printf ("TRESFIB: image file %s read\n")
		    call pargstr (imf0)
		call flush (STDOUT)
		}

# Set fiber size from spectrum header FIBSIZE keyword
	    call imgspar (im, "FIBSIZE", fibsize, 8)
	    if (strcmp (fibsize, "1.4") == 0) {
		call strcpy ("s", apfib, 7)
		call strcpy ("s", fibcode, 7)
		}
	    else if (strcmp (fibsize, "2.3") == 0) {
		call strcpy ("m", apfib, 7)
		call strcpy ("m", fibcode, 7)
		}
	    else if (strcmp (fibsize, "3.2") == 0) {
		call strcpy ("l", apfib, 7)
		call strcpy ("l", fibcode, 7)
		}

# Set binning from spectrum header CCDSUM keyword
	    call imgspar (im, "CCDSUM", ccdsum, 8)
	    if (ccdsum[1] == '0') {
		call strcpy (imf, imf1, SZ_FNAME)
		call strcat ("[1]", imf1, SZ_FNAME)
		iferr (im1 = immap (imf1, READ_ONLY, 0)) {
		    if (im1 != NULL)
			call imunmap (im1)
		    if (db) {
			call printf ("TRESFIB: image file %s read error\n")
			    call pargstr (imf1)
			call flush (STDOUT)
			}
		    return
		    }
		else if (db) {
		    call printf ("TRESFIB: image file %s read\n")
		    call pargstr (imf1)
		    call flush (STDOUT)
		    }
		call imgspar (im1, "CCDSUM", ccdsum, 8)
		}
	    if (ccdsum[1] == '2') {
		binning = 2
		call strcat ("b", apfib, 7)
		}
	    else
		binning = 1

# Set fiber number from spectrum header FIBER keyword, if it is there
	    fiber[1] = EOS
	    call imgspar (im, "FIBER", fiber, 8)
	    if (strlen (fiber) > 0) {
		call strcat (fiber, apfib, 7)
		call strcat (fiber, apertures, 7)
		}

# Otherwise set fiber number(s) from spectrum header FIBKEY keyword
	    else {
		call imgspar (im, "FIBKEY", fibkey, 8)
		if (fibkey[1] != 'N') {
		    call strcat ("1", apfib, 7)
		    call strcat ("1", apertures, 7)
		    }
		if (fibkey[2] != 'N') {
		    call strcat ("2", apfib, 7)
		    call strcat ("2", apertures, 7)
		    }
		if (fibkey[3] != 'N') {
		    call strcat ("3", apfib, 7)
		    call strcat ("2", apertures, 7)
		    }
		}
	    }

# If image is not specified, set apfib from task parameters
	else {

# Set fiber size from parameter fibcode
	    call clgstr ("fibcode", fibcode, 7)
	    if (stridx ('s', fibcode) > 0) {
		call strcpy ("s", apfib, 7)
		call strcpy ("1.4", fibsize)
		}
	    else if (stridx ('m', fibcode) > 0) {
		call strcpy ("m", apfib, 7)
		call strcpy ("2.3", fibsize)
		}
	    else if (stridx ('l', fibcode) > 0) {
		call strcpy ("l", apfib, 7)
		call strcpy ("3.2", fibsize)
		}
	    else {
		call clgstr ("fibsize", fibsize, 7)
		if (strcmp (fibsize, "1.4") == 0) {
		    call strcpy ("s", apfib, 7)
		    call strcpy ("s", fibcode, 7)
		    }
		else if (strcmp (fibsize, "2.3") == 0) {
		    call strcpy ("m", apfib, 7)
		    call strcpy ("m", fibcode, 7)
		    }
		else if (strcmp (fibsize, "3.2") == 0) {
		    call strcpy ("l", apfib, 7)
		    call strcpy ("l", fibcode, 7)
		    }
		}

# Set binning from parameter binning
	    call clgstr ("binning", binning, 7)
	    if (stridx ('2', binning) > 0)
		call strcat ("b", apfib,7)

# Set fiber from parameter apertures
	    call clgstr ("apertures", apertures, 7)
	    if (stridx ('1', apertures) > 0)
		call strcat ("1", apfib)
	    if (stridx ('2', apertures) > 0)
		call strcat ("2", apfib)
	    if (stridx ('3', apertures) > 0)
		call strcat ("3", apfib)
	    }

	if (vb) {
	    call printf ("TRSDECODE apertures=%s, size=%s(%s), binning=%d -> %s\n")
		call pargstr (apertures)
		call pargstr (fibcode)
		call pargstr (fibsize)
		call pargi (binning)
		call pargstr (apfib)
	    }
	
# Write spectrum parameters to task parameter file
	call clpstr ("apfib", apfib, 7)

end

# May 20 2008	New program
